"""

Copyright (c) 2020, Vanessa Sochat

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""
from rest_framework.authtoken.models import Token


def get_usertoken(user):
    try:
        token = Token.objects.get(user=user)
    except Token.DoesNotExist:
        token = Token.objects.create(user=user)
    return token.key


def get_token(request):
    """The same as validate_token, but return the token object to check the
    associated user.
    """
    # Coming from HTTP, look for authorization as bearer token
    token = request.META.get("HTTP_AUTHORIZATION")

    if token:
        try:
            return Token.objects.get(key=token.replace("BEARER", "").strip())
        except Token.DoesNotExist:
            pass

    # Next attempt - try to get token via user session
    elif request.user.is_authenticated and not request.user.is_anonymous:
        try:
            return Token.objects.get(user=request.user)
        except Token.DoesNotExist:
            pass


def get_challenge(repository, scopes=["pull", "push"]):
    """Given an unauthenticated request, return a challenge in
    the Www-Authenticate header
    """
    if not isinstance(scopes, list):
        scopes = [scopes]
    return 'realm="%s/auth/token",service="%s",scope="repository:%s:%s"' % (
        settings.DOMAIN_NAME,
        settings.DOMAIN_NAME,
        repository,
        ",".join(scopes),
    )



# Generated by Django 3.1.1 on 2020-10-11 03:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0004_auto_20200920_2322"),
    ]

    operations = [
        migrations.AlterField(
            model_name="image",
            name="manifest",
            field=models.BinaryField(default=b"{}"),
        ),
    ]



"""

Copyright (c) 2020, Vanessa Sochat

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.renderers import JSONRenderer, BrowsableAPIRenderer
from django.http.response import Http404, HttpResponse
from django.views.decorators.cache import never_cache

from django_oci.models import Repository, Image, get_image_by_tag
from django_oci import settings
from django_oci.storage import storage
from .parsers import ManifestRenderer


import os

# from opencontainers.image.v1.manifest import Manifest
# from rest_framework import generics, serializers, viewsets, status
# from rest_framework.exceptions import PermissionDenied, NotFound
# from ratelimit.mixins import RatelimitMixin


class ImageTags(APIView):
    """Return a list of tags for an image."""

    permission_classes = []
    allowed_methods = ("GET",)

    @never_cache
    def get(self, request, *args, **kwargs):
        """GET /v2/<name>/tags/list"""
        name = kwargs.get("name")
        number = request.GET.get("n")
        last = request.GET.get("last")

        try:
            repository = Repository.objects.get(name=name)
        except Repository.DoesNotExist:
            raise Http404

        tags = [
            x
            for x in list(repository.image_set.values_list("tag__name", flat=True))
            if x
        ]

        # Tags must be sorted in lexical order
        tags.sort()

        # Number must be an integer if defined
        if number:
            number = int(number)

        # if last, <tagname> not included in the results, but up to <int> tags after <tagname> will be returned.
        if last and number:
            try:
                start = tags.index(last)
            except IndexError:
                start = 0
            tags = tags[start:number]

        elif number:
            tags = tags[:number]

        # Ensure tags sorted in lexical order
        data = {"name": repository.name, "tags": sorted(tags)}
        return Response(status=200, data=data)


class ImageManifest(APIView):
    """An Image Manifest holds the configuration and metadata about an image
    GET: is to retrieve an existing image manifest
    PUT: is to push a manifest
    """

    renderer_classes = [ManifestRenderer, JSONRenderer]
    permission_classes = []
    allowed_methods = (
        "GET",
        "PUT",
        "DELETE",
    )

    @never_cache
    def delete(self, request, *args, **kwargs):
        """DELETE /v2/<name>/manifests/<tag>"""

        # A registry must globally disable or enable both
        if settings.DISABLE_TAG_MANIFEST_DELETE:
            return Response(status=405)

        name = kwargs.get("name")
        reference = kwargs.get("reference")
        tag = kwargs.get("tag")

        from django_oci.models import Tag

        # Retrieve the image, return of None indicates not found
        image = get_image_by_tag(name, reference=reference, tag=tag, create=False)
        if not image:
            raise Http404

        # Delete the image tag
        if tag:
            tag = image.tag_set.filter(name=tag)
            tag.delete()

        # Delete a manifest
        elif reference:
            image.delete()

        # Upon success, the registry MUST respond with a 202 Accepted code.
        return Response(status=202)

    @never_cache
    def put(self, request, *args, **kwargs):
        """PUT /v2/<name>/manifests/<reference>
        https://github.com/opencontainers/distribution-spec/blob/master/spec.md#pushing-manifests
        """
        # We likely can default to the v1 manifest, unless otherwise specified
        # application/vnd.oci.image.manifest.v1+json
        content_type = request.META.get(
            "CONTENT_TYPE", settings.IMAGE_MANIFEST_CONTENT_TYPE
        )

        name = kwargs.get("name")
        reference = kwargs.get("reference")
        tag = kwargs.get("tag")

        # Reference must be sha256 digest
        if reference and not reference.startswith("sha256:"):
            return Response(status=400)

        # Also provide the body in case we have a tag
        image = get_image_by_tag(name, reference, tag, create=True, body=request.body)
        return Response(status=201, headers={"Location": image.get_manifest_url()})

    @never_cache
    def get(self, request, *args, **kwargs):
        """GET /v2/<name>/manifests/<reference>"""

        name = kwargs.get("name")
        reference = kwargs.get("reference")
        tag = kwargs.get("tag")

        image = get_image_by_tag(name, tag=tag, reference=reference)

        # If the manifest is not found in the registry, the response code MUST be 404 Not Found.
        if not image:
            raise Http404
        return Response(image.manifest, status=200)



"""

Copyright (c) 2020, Vanessa Sochat

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""
from django.db.models.signals import post_delete
from django.dispatch import receiver
from .models import Image, Blob
import os


@receiver(post_delete, sender=Image)
def delete_blobs(sender, instance, **kwargs):
    print("Delete image signal running.")

    for blob in instance.blobs.all():
        if hasattr(blob, "datafile"):
            if blob.image_set.count() == 0:
                print("Deleting %s, no longer used." % blob.datafile)
                blob.datafile.delete()
                blob.delete()



class SingleContainerSerializer(serializers.ModelSerializer):

    collection = serializers.SerializerMethodField("collection_name")
    image = serializers.SerializerMethodField("get_download_url")
    metadata = serializers.SerializerMethodField("get_cleaned_metadata")

    def collection_name(self, container):
        return container.collection.name

    def get_cleaned_metadata(self, container):
        metadata = container.metadata
        for key in ["build_metadata", "builder", "build_finish", "image"]:
            if key in metadata:
                del metadata[key]
        return metadata

    def get_download_url(self, container):

        secret = container.collection.secret
        download_url = reverse(
            "download_container", kwargs={"cid": container.id, "secret": secret}
        )

        return "%s%s" % (settings.DOMAIN_NAME, download_url)

    class Meta:
        model = Container
        fields = (
            "id",
            "name",
            "image",
            "tag",
            "add_date",
            "metrics",
            "version",
            "tag",
            "frozen",
            "metadata",
            "collection",
        )


################################################################################
# Multiple Object Serializers
################################################################################


class ContainerSerializer(serializers.HyperlinkedModelSerializer):

    collection = serializers.SerializerMethodField("collection_name")
    metadata = serializers.SerializerMethodField("get_cleaned_metadata")

    def collection_name(self, container):
        return container.collection.name

    def get_cleaned_metadata(self, container):
        metadata = container.metadata
        for key in ["build_metadata", "builder", "build_finish", "image"]:
            if key in metadata:
                del metadata[key]
        return metadata

    class Meta:
        model = Container
        fields = (
            "id",
            "name",
            "tag",
            "add_date",
            "metrics",
            "version",
            "tag",
            "frozen",
            "metadata",
            "collection",
        )

    id = serializers.ReadOnlyField()


################################################################################
# ViewSets: requests for (paginated) information about containers
################################################################################


class ContainerViewSet(viewsets.ReadOnlyModelViewSet):
    """View all containers"""

    def get_queryset(self):
        return Container.objects.filter(collection__private=False)

    serializer_class = ContainerSerializer


################################################################################
# Container Views: custom views for specific containers
################################################################################


class ContainerDetailByName(LoggingMixin, RatelimitMixin, generics.GenericAPIView):
    """Retrieve a container instance based on it's name"""

    ratelimit_key = "ip"
    ratelimit_rate = settings.VIEW_RATE_LIMIT
    ratelimit_block = settings.VIEW_RATE_LIMIT_BLOCK
    ratelimit_method = "GET"

    def get_object(
        self, collection, name, tag=None, version=None
    ):  # pylint: disable=arguments-differ

        try:

            # Given collection, container, tag and version
            if tag is not None and version is not None:
                container = Container.objects.get(
                    collection__name=collection, name=name, tag=tag, version=version
                )

            # Given collection, container, version
            elif tag is None:
                container = Container.objects.get(
                    collection__name=collection, name=name, version=version
                )
            # Given collection, container, tag
            elif version is None:
                container = Container.objects.get(
                    collection__name=collection, name=name, tag=tag
                )
            # Given collection, container
            else:
                container = Container.objects.get(
                    collection__name=collection, name=name
                )
        except Container.DoesNotExist:
            container = None
        return container

    def delete(self, request, collection, name, tag=None, version=None):
        from shub.apps.api.actions import delete_container

        container = self.get_object(
            collection=collection, name=name, tag=tag, version=version
        )

        if container is None:
            full_name = "%s/%s" % (collection, name)
            container = self.get_object(collection=full_name, name=full_name, tag=tag)
        if container is None:
            raise NotFound(detail="Container Not Found")

        if container.frozen is True:
            message = "%s is frozen, delete not allowed." % container.get_short_uri()
            raise PermissionDenied(detail=message, code=304)

        # This only deletes container object, not remote builds.
        if delete_container(request, container):
            container.delete()
            return Response(status=status.HTTP_204_NO_CONTENT)

        raise PermissionDenied(detail="Unauthorized")

    def get(self, request, collection, name, tag="latest", version=None):
        container = self.get_object(collection=collection, name=name, tag=tag)

        # if container None, likely is google build (container name includes collection)
        if container is None:
            full_name = "%s/%s" % (collection, name)
            container = self.get_object(collection=full_name, name=full_name, tag=tag)

        return _container_get(request, container, name, tag)


def _container_get(request, container, name=None, tag=None):
    """container get is the shared function for getting a container based
    on a name or an id. It validates the request and returns a response.

    Parameters
    ==========
    request: the request from the view with the user
    container: the container object to check
    """
    if container is None:
        raise NotFound

    if name is None:
        name = container.name

    if tag is None:
        tag = container.tag

    # The user isn't allowed to get more than the limit
    if container.get_count >= container.get_limit:
        return Response(429)

    if container.collection.get_count >= container.collection.get_limit:
        return Response(429)

    # All public images are pull-able

    is_private = container.collection.private

    if not is_private:
        serializer = SingleContainerSerializer(container)
        return Response(serializer.data)

    # Determine if user has permission to get if private
    auth = request.META.get("HTTP_AUTHORIZATION")

    if auth is None:
        print("Auth is None")
        raise PermissionDenied(detail="Authentication Required")

    # Validate User Permissions - must have view to pull private image

    if not has_permission(auth, container.collection):
        print("Does not have permission")
        raise PermissionDenied(detail="Unauthorized")

    timestamp = generate_timestamp()
    payload = "pull|%s|%s|%s|%s|" % (container.collection.name, timestamp, name, tag)

    if validate_request(auth, payload, "pull", timestamp):
        serializer = SingleContainerSerializer(container)
        return Response(serializer.data)

    return Response(400)


# SERIALIZERS

# Collections


class CollectionSerializer(serializers.ModelSerializer):
    """provide all fields, including notes"""

    tags = serializers.PrimaryKeyRelatedField(
        queryset=Tag.objects.all(), required=False, many=True
    )
    parent = serializers.PrimaryKeyRelatedField(
        queryset=Collection.objects.all(), required=False
    )
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Collection
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "parent",
            "tags",
            "label",
        )


class CollectionViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Collection.objects.all()

    serializer_class = CollectionSerializer
    permission_classes = (AllowAnyGet,)


# Composite Part


class CompositePartSerializer(serializers.ModelSerializer):

    label = serializers.SerializerMethodField("get_label")
    parts = serializers.PrimaryKeyRelatedField(many=True, queryset=Part.objects.all())

    def get_label(self, instance):
        return "compositepart"

    class Meta:
        model = CompositePart

        # Extra keyword arguments for create
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "composite_id",
            "composite_type",
            "direction_string",
            "sequence",
            "parts",
            "label",
        )


class CompositePartViewSet(viewsets.ModelViewSet):
    permission_classes = (AllowAnyGet,)

    def get_queryset(self):
        return CompositePart.objects.all()

    def create(self, request, *args, **kwargs):
        """create a new composite part! We require existing part ids, along
        with a name and direction string. The client should already have
        handled doing the processing to derive the direction string
        and part ids, we just validate that the parts are in the sequence.
        """
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        # We need at least one part
        parts = serializer.validated_data.get("parts")

        if len(parts) < 2:
            raise serializers.ValidationError(
                "You must provide more than one part for a composite part"
            )

        # If no direction string provided, create default
        direction_string = serializer.validated_data.get("direction_string")
        sequence = serializer.validated_data.get("sequence")

        # Validate that length of part ids == length of direction string
        if len(parts) != len(direction_string):
            raise serializers.ValidationError(
                "Direction string must be equal length to number of parts provided."
            )

        # Ensure that each part is at least included in the sequence
        for i, part in enumerate(parts):
            direction = direction_string[i]

            # Check if forward and reverse isn't there.
            if (
                part.optimized_sequence not in sequence
                and part.optimized_sequence[::-1] not in sequence
            ):
                raise serializers.ValidationError(
                    "%s with direction %s not in sequence" % (part.uuid, direction)
                )

        # Then perform create
        self.perform_create(serializer, sequence)
        headers = self.get_success_headers(serializer.data)
        return Response(
            serializer.data, status=status.HTTP_201_CREATED, headers=headers
        )

    def perform_create(self, serializer, sequence):
        serializer.save(sequence=sequence)

    serializer_class = CompositePartSerializer


# Distributions


class DistributionSerializer(serializers.ModelSerializer):

    platesets = serializers.PrimaryKeyRelatedField(
        many=True, queryset=PlateSet.objects.all()
    )
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Distribution
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "platesets",
            "label",
        )


class DistributionViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Distribution.objects.all()

    serializer_class = DistributionSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Modules


class ModuleSerializer(serializers.ModelSerializer):
    """a Module serializer provides all fields except for data and notes. The
    user is required to use the SingleModuleSerializer to get the extra data
    """

    container = serializers.PrimaryKeyRelatedField(queryset=Container.objects.all())
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Module
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "container",
            "notes",
            "model_id",
            "module_type",
            "data",
            "label",
        )


class ModuleViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Module.objects.all()

    serializer_class = ModuleSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Institutions


class InstitutionSerializer(serializers.ModelSerializer):

    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Institution
        fields = ("uuid", "name", "signed_master", "label")


class InstitutionViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Institution.objects.all()

    serializer_class = InstitutionSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Operations


class OperationSerializer(serializers.ModelSerializer):

    plans = serializers.PrimaryKeyRelatedField(
        queryset=Plan.objects.all(), required=False, many=True
    )
    label = serializers.SerializerMethodField("get_label")

    def plans_list(self, operation):
        plans = []
        for plan in operation.plans.all():
            plans.append({"name": plan.name, "uuid": plan.uuid})
        return plans

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Operation
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "plans",
            "label",
        )


class OperationViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Operation.objects.all()

    serializer_class = OperationSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Orders


class OrderSerializer(serializers.ModelSerializer):

    distributions = serializers.PrimaryKeyRelatedField(
        queryset=Distribution.objects.all(), required=False, many=True
    )
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Order
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "notes",
            "distributions",
            "label",
        )


class OrderViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Order.objects.all()

    serializer_class = OrderSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Organisms


class OrganismSerializer(serializers.ModelSerializer):

    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Organism
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "genotype",
            "label",
        )


class OrganismViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Organism.objects.all()

    serializer_class = OrganismSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Part


class PartSerializer(serializers.ModelSerializer):
    """a part serializer exposes all fields, meaning the user has
    looked up a part based on a uuid.
    """

    tags = serializers.PrimaryKeyRelatedField(
        queryset=Tag.objects.all(), required=False, many=True
    )
    collections = serializers.PrimaryKeyRelatedField(
        queryset=Collection.objects.all(), required=False, many=True
    )
    author = serializers.PrimaryKeyRelatedField(queryset=Author.objects.all())
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Part
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "status",
            "gene_id",
            "part_type",
            "genbank",
            "original_sequence",
            "optimized_sequence",
            "synthesized_sequence",
            "full_sequence",
            "vector",
            "primer_forward",
            "primer_reverse",
            "barcode",
            "label",
            "translation",
            "tags",
            "collections",
            "author",
        )


class PartViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Part.objects.all()

    serializer_class = PartSerializer
    permission_classes = (AllowAnyGet,)


# Plans


class PlanSerializer(serializers.ModelSerializer):

    parent = serializers.PrimaryKeyRelatedField(
        queryset=Plan.objects.all(), required=False
    )
    operation = serializers.PrimaryKeyRelatedField(queryset=Operation.objects.all())
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Plan
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "parent",
            "operation",
            "status",
            "label",
        )


class PlanViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Plan.objects.all()

    serializer_class = PlanSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Plates


class PlateSerializer(serializers.ModelSerializer):

    container = serializers.PrimaryKeyRelatedField(queryset=Container.objects.all())
    protocol = serializers.PrimaryKeyRelatedField(
        queryset=Protocol.objects.all(), required=False
    )
    wells = serializers.PrimaryKeyRelatedField(
        queryset=Well.objects.all(), many=True, required=False
    )
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Plate
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "plate_type",
            "plate_form",
            "status",
            "name",
            "thaw_count",
            "notes",
            "height",
            "length",
            "container",
            "protocol",
            "wells",
            "label",
            "plate_vendor_id",
        )


class PlateViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Plate.objects.all()

    serializer_class = PlateSerializer
    permission_classes = (AllowAnyGet,)


# PlateSet


class PlateSetSerializer(serializers.ModelSerializer):
    """platesets serializers"""

    plates = serializers.PrimaryKeyRelatedField(queryset=Plate.objects.all(), many=True)
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = PlateSet
        fields = (
            "uuid",
            "description",
            "name",
            "time_created",
            "time_updated",
            "plates",
            "label",
        )


class PlateSetViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return PlateSet.objects.all()

    serializer_class = PlateSetSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Protocol


class ProtocolSerializer(serializers.ModelSerializer):

    schema = serializers.PrimaryKeyRelatedField(
        queryset=Schema.objects.all(), required=False
    )
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Protocol
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "data",
            "description",
            "label",
            "schema",
        )


class ProtocolViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Protocol.objects.all()

    serializer_class = ProtocolSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Robot


class RobotSerializer(serializers.ModelSerializer):

    left_mount = serializers.PrimaryKeyRelatedField(queryset=Module.objects.all())
    right_mount = serializers.PrimaryKeyRelatedField(queryset=Module.objects.all())
    container = serializers.PrimaryKeyRelatedField(queryset=Container.objects.all())
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Robot
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "container",
            "name",
            "robot_id",
            "robot_type",
            "notes",
            "server_version",
            "right_mount",
            "left_mount",
            "label",
        )


class RobotViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Robot.objects.all()

    serializer_class = RobotSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Sample


class SampleSerializer(serializers.ModelSerializer):

    part = serializers.PrimaryKeyRelatedField(queryset=Part.objects.all())
    derived_from = serializers.PrimaryKeyRelatedField(
        queryset=Sample.objects.all(), required=False
    )
    wells = serializers.PrimaryKeyRelatedField(queryset=Well.objects.all(), many=True)
    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Sample
        fields = (
            "uuid",
            "outside_collaborator",
            "sample_type",
            "status",
            "evidence",
            "vendor",
            "time_created",
            "time_updated",
            "derived_from",
            "part",
            "index_forward",
            "index_reverse",
            "label",
            "wells",
        )


class SampleViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Sample.objects.all()

    serializer_class = SampleSerializer
    permission_classes = (AllowAnyGet,)


# Schema


class SchemaSerializer(serializers.ModelSerializer):

    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Schema
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "description",
            "schema",
            "schema_version",
            "label",
        )


class SchemaViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Schema.objects.all()

    serializer_class = SchemaSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Tags


class TagSerializer(serializers.ModelSerializer):

    label = serializers.SerializerMethodField("get_label")

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Tag
        fields = ("uuid", "tag", "label")


class TagViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Tag.objects.all()

    serializer_class = TagSerializer
    permission_classes = (IsStaffOrSuperUser,)


# Wells


class WellSerializer(serializers.ModelSerializer):

    label = serializers.SerializerMethodField("get_label")
    organism = serializers.SerializerMethodField("get_organism")

    def get_organism(self, instance):
        return OrganismSerializer(instance.organism).data

    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Well
        fields = (
            "uuid",
            "address",
            "volume",
            "quantity",
            "media",
            "time_created",
            "time_updated",
            "organism",
            "label",
        )



# Generated by Django 3.1.1 on 2020-09-19 20:41

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_oci.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Blob",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "add_date",
                    models.DateTimeField(auto_now_add=True, verbose_name="date added"),
                ),
                (
                    "modify_date",
                    models.DateTimeField(auto_now=True, verbose_name="date modified"),
                ),
                ("content_type", models.CharField(max_length=250)),
                ("digest", models.CharField(blank=True, max_length=250, null=True)),
                (
                    "datafile",
                    models.FileField(
                        max_length=255, upload_to=django_oci.models.get_upload_folder
                    ),
                ),
                ("remotefile", models.CharField(blank=True, max_length=500, null=True)),
            ],
        ),
        migrations.CreateModel(
            name="Image",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "add_date",
                    models.DateTimeField(
                        auto_now=True, verbose_name="date manifest added"
                    ),
                ),
                ("manifest", models.TextField(default="{}")),
                ("version", models.CharField(blank=True, max_length=250, null=True)),
                ("blobs", models.ManyToManyField(to="django_oci.Blob")),
            ],
        ),
        migrations.CreateModel(
            name="Tag",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=250)),
                (
                    "image",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="django_oci.image",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Repository",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=500, unique=True)),
                (
                    "add_date",
                    models.DateTimeField(auto_now_add=True, verbose_name="date added"),
                ),
                (
                    "modify_date",
                    models.DateTimeField(auto_now=True, verbose_name="date modified"),
                ),
                (
                    "private",
                    models.BooleanField(
                        choices=[
                            (
                                False,
                                "Public (The collection will be accessible by anyone)",
                            ),
                            (True, "Private (The collection will be not listed.)"),
                        ],
                        default=django_oci.models.get_privacy_default,
                        verbose_name="Accessibility",
                    ),
                ),
                (
                    "contributors",
                    models.ManyToManyField(
                        blank=True,
                        help_text="users with edit permission to the collection",
                        related_name="container_collection_contributors",
                        related_query_name="contributor",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="Contributors",
                    ),
                ),
                (
                    "owners",
                    models.ManyToManyField(
                        blank=True,
                        default=None,
                        related_name="container_collection_owners",
                        related_query_name="owners",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
        ),
        migrations.AddField(
            model_name="image",
            name="repository",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="django_oci.repository"
            ),
        ),
        migrations.AddField(
            model_name="blob",
            name="repository",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="django_oci.repository"
            ),
        ),
        migrations.AlterUniqueTogether(
            name="image",
            unique_together={("repository", "version")},
        ),
        migrations.CreateModel(
            name="Annotation",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("key", models.CharField(max_length=250)),
                ("value", models.CharField(max_length=250)),
                (
                    "images",
                    models.ManyToManyField(
                        related_name="annotations", to="django_oci.Image"
                    ),
                ),
            ],
            options={
                "unique_together": {("key", "value")},
            },
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-20 22:37

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0002_auto_20200920_0155"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="annotation",
            name="images",
        ),
        migrations.AddField(
            model_name="annotation",
            name="image",
            field=models.ForeignKey(
                default=None,
                on_delete=django.db.models.deletion.CASCADE,
                to="django_oci.image",
            ),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.1.1 on 2020-09-20 01:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0001_initial"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="blob",
            unique_together={("repository", "digest")},
        ),
    ]



# Generated by Django 3.1.1 on 2020-10-12 04:38

from django.db import migrations, models
import django_oci.models


class Migration(migrations.Migration):

    dependencies = [
        ("django_oci", "0005_auto_20201011_0325"),
    ]

    operations = [
        migrations.AlterField(
            model_name="blob",
            name="datafile",
            field=models.FileField(
                max_length=255,
                storage=django_oci.models.OverwriteStorage(),
                upload_to=django_oci.models.get_upload_folder,
            ),
        ),
    ]



"""

Copyright (c) 2020, Vanessa Sochat

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from django.conf import settings
from django.urls import reverse

from django_oci.models import Repository, Image
from .permissions import IsStaffOrSuperUser, AllowAnyGet
from rest_framework import generics, mixins, serializers, viewsets, status
from rest_framework.exceptions import PermissionDenied, NotFound

from rest_framework.response import Response
from rest_framework.views import APIView

# Repository


class RepositorySerializer(serializers.ModelSerializer):
    def get_label(self, instance):
        return instance.get_label()

    class Meta:
        model = Repository
        fields = ("uuid", "name", "email", "affiliation", "orcid", "tags", "label")


class RepositoryViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Repository.objects.all()

    serializer_class = RepositorySerializer
    permission_classes = (AllowAnyGet,)


# Image


class ImageSerializer(serializers.ModelSerializer):

    plates = serializers.SerializerMethodField("plates_list")
    parent = serializers.PrimaryKeyRelatedField(queryset=Image.objects.all())
    label = serializers.SerializerMethodField("get_label")

    def plates_list(self, container):
        return [
            {"name": plate.name, "uuid": plate.uuid}
            for plate in container.plate_set.all()
        ]

    def get_label(self, container):
        return container.get_label()

    class Meta:
        model = Image
        fields = (
            "uuid",
            "time_created",
            "time_updated",
            "name",
            "container_type",
            "description",
            "estimated_temperature",
            "x",
            "y",
            "z",
            "parent",
            "plates",
            "label",
        )


class ImageViewSet(viewsets.ModelViewSet):
    def get_queryset(self):
        return Image.objects.all()

    serializer_class = ImageSerializer
    permission_classes = (IsStaffOrSuperUser,)



# -*- coding: utf-8 -*-

from rest_framework.views import APIView
from rest_framework.response import Response

# from rest_framework import generics, serializers, viewsets, status
# from rest_framework.exceptions import PermissionDenied, NotFound
# from ratelimit.mixins import RatelimitMixin


class APIVersionCheck(APIView):
    """provide version support information based on response statuses.
    This endpoint should only allow GET requests.
    https://github.com/opencontainers/distribution-spec/blob/master/spec.md#api-version-check
    """

    permission_classes = []
    allowed_methods = ("GET",)

    def get(self, request, *args, **kwargs):
        headers = {"Docker-Distribution-API-Version": "registry/2.0"}
        return Response({"success": True})



"""

Copyright (c) 2020, Vanessa Sochat

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

"""

from rest_framework.permissions import BasePermission, SAFE_METHODS


class IsStaffOrSuperUser(BasePermission):
    """Allows access to staff (admin) or superuser."""

    def has_permission(self, request, view):

        if request.user.is_staff or request.user.is_superuser:
            return True

        return request.method in SAFE_METHODS


class AllowAnyGet(BasePermission):
    """Allows an anonymous user access for GET requests only."""

    def has_permission(self, request, view):

        if request.user.is_anonymous and request.method == "GET":
            return True

        if request.user.is_staff or request.user.is_superuser:
            return True

        return request.method in SAFE_METHODS



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

def get_version(*file_paths):
    """Retrieves the version from django_oci/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("django_oci", "__init__.py")

with open('README.md') as fd:
    readme = fd.read()

setup(
    name='django-oci',
    version=version,
    description="""Open Containers distribution API for Django""",
    long_description=readme,
    long_description_content_type="text/markdown",
    author='Vanessa Sochat',
    author_email='vsochat@stanford.edu',
    url='https://github.com/vsoch/django-oci',
    packages=[
        'django_oci',
    ],
    include_package_data=True,
    install_requires=["djangorestframework"],
    license="Apache Software License 2.0",
    zip_safe=False,
    keywords='django-oci',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
